@@ -47,6 +47,7 @@
 #include "empathy-avatar-chooser.h"
 #include "empathy-avatar-image.h"
 #include "empathy-ui-utils.h"
+#include "empathy-string-parser.h"
 #include "empathy-kludge-label.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
@@ -81,6 +82,7 @@ typedef struct
   EmpathyContact *contact;
   EmpathyContactWidgetFlags flags;
   guint widget_id_timeout;
+  gulong fav_sig_id;
 
   GtkWidget *vbox_contact_widget;
 
@@ -99,6 +101,7 @@ typedef struct
   GtkWidget *label_status;
   GtkWidget *table_contact;
   GtkWidget *vbox_avatar;
+  GtkWidget *favourite_checkbox;
 
   /* Location */
   GtkWidget *vbox_location;
@@ -447,7 +450,6 @@ contact_widget_groups_setup (EmpathyContactWidget *information)
 {
   if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_GROUPS)
     {
-      information->manager = empathy_contact_manager_dup_singleton ();
       contact_widget_model_setup (information);
     }
 }
@@ -600,6 +602,7 @@ contact_widget_location_update (EmpathyContactWidget *information)
 
       text = g_strconcat ( _("<b>Location</b>, "), user_date, NULL);
       gtk_label_set_markup (GTK_LABEL (information->label_location), text);
+      g_free (user_date);
       g_free (text);
     }
 
@@ -948,14 +951,36 @@ contact_widget_name_notify_cb (EmpathyContactWidget *information)
 static void
 contact_widget_presence_notify_cb (EmpathyContactWidget *information)
 {
-  gtk_label_set_text (GTK_LABEL (information->label_status),
-      empathy_contact_get_status (information->contact));
+  const gchar *status;
+  gchar *markup_text = NULL;
+
+  status = empathy_contact_get_status (information->contact);
+  if (status != NULL)
+    markup_text = empathy_add_link_markup (status);
+  gtk_label_set_markup (GTK_LABEL (information->label_status), markup_text);
+  g_free (markup_text);
+
   gtk_image_set_from_icon_name (GTK_IMAGE (information->image_state),
       empathy_icon_name_for_contact (information->contact),
       GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (information->image_state);
 }
 
+#if HAVE_FAVOURITE_CONTACTS
+static void
+contact_widget_favourites_changed_cb (EmpathyContactManager *manager,
+    EmpathyContact *contact,
+    gboolean is_favourite,
+    EmpathyContactWidget *information)
+{
+  if (contact != information->contact)
+    return;
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
+            information->favourite_checkbox), is_favourite);
+}
+#endif
+
 static void
 contact_widget_remove_contact (EmpathyContactWidget *information)
 {
@@ -1044,6 +1069,20 @@ contact_widget_contact_update (EmpathyContactWidget *information)
       contact_widget_presence_notify_cb (information);
       contact_widget_avatar_notify_cb (information);
 
+#if HAVE_FAVOURITE_CONTACTS
+      if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_FAVOURITE)
+        {
+          gboolean is_favourite;
+
+          is_favourite = empathy_contact_list_is_favourite (
+              EMPATHY_CONTACT_LIST (information->manager),
+              information->contact);
+
+          contact_widget_favourites_changed_cb (information->manager,
+              information->contact, is_favourite, information);
+        }
+#endif
+
       gtk_widget_show (information->label_alias);
       gtk_widget_show (information->widget_alias);
       gtk_widget_show (information->hbox_presence);
@@ -1173,6 +1212,28 @@ contact_widget_id_focus_out_cb (GtkWidget *widget,
   return FALSE;
 }
 
+#if HAVE_FAVOURITE_CONTACTS
+static void
+favourite_toggled_cb (GtkToggleButton *button,
+    EmpathyContactWidget *information)
+{
+  gboolean active;
+
+  active = gtk_toggle_button_get_active (button);
+
+  if (active)
+    {
+      empathy_contact_list_add_to_favourites (
+          EMPATHY_CONTACT_LIST (information->manager), information->contact);
+    }
+  else
+    {
+      empathy_contact_list_remove_from_favourites (
+          EMPATHY_CONTACT_LIST (information->manager), information->contact);
+    }
+}
+#endif
+
 static void
 contact_widget_contact_setup (EmpathyContactWidget *information)
 {
@@ -1310,6 +1371,27 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
     gtk_label_set_selectable (GTK_LABEL (information->label_status), FALSE);
   }
   gtk_widget_show (information->widget_alias);
+
+#if HAVE_FAVOURITE_CONTACTS
+  /* Favorite */
+  if (information->flags & EMPATHY_CONTACT_WIDGET_EDIT_FAVOURITE)
+    {
+      information->favourite_checkbox = gtk_check_button_new_with_label (
+          _("Favorite"));
+
+      g_signal_connect (information->favourite_checkbox, "toggled",
+          G_CALLBACK (favourite_toggled_cb), information);
+
+      gtk_table_attach_defaults (GTK_TABLE (information->table_contact),
+           information->favourite_checkbox, 0, 2, 3, 4);
+
+      information->fav_sig_id = g_signal_connect (information->manager,
+          "favourites-changed",
+          G_CALLBACK (contact_widget_favourites_changed_cb), information);
+
+      gtk_widget_show (information->favourite_checkbox);
+    }
+#endif
 }
 
 static void
@@ -1322,10 +1404,11 @@ contact_widget_destroy_cb (GtkWidget *widget,
     {
       g_source_remove (information->widget_id_timeout);
     }
-  if (information->manager)
-    {
-      g_object_unref (information->manager);
-    }
+
+  if (information->fav_sig_id != 0)
+    g_signal_handler_disconnect (information->manager, information->fav_sig_id);
+
+  g_object_unref (information->manager);
 
   g_slice_free (EmpathyContactWidget, information);
 }
@@ -1393,6 +1476,8 @@ empathy_contact_widget_new (EmpathyContact *contact,
       "EmpathyContactWidget",
       information);
 
+  information->manager = empathy_contact_manager_dup_singleton ();
+
   /* Create widgets */
   contact_widget_contact_setup (information);
   contact_widget_groups_setup (information);
